/* Copyright (c) 2015 Alinson Xavier
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <time.h>
#include "greedy-2d.h"
#include "util.h"
#include "double.h"

int RANDOMIZER_generate_triangle(double *f,
                                 double *vertices,
                                 double *lattice_points)
{
    int rval = 0;

    srand((unsigned int) time(0));

    f[0] = DOUBLE_random(0.0, 1.0);
    f[1] = DOUBLE_random(0.0, 1.0);

    double rays[6] = {
            DOUBLE_random(0, 1000.0), DOUBLE_random(0, 1000.0),
            DOUBLE_random(-1000.0, 0), DOUBLE_random(0, 1000.0),
            0, DOUBLE_random(-1000.0, 0)
    };

    log_verbose("r1=%12.8lf %12.8lf\n", rays[0], rays[1]);
    log_verbose("r2=%12.8lf %12.8lf\n", rays[2], rays[3]);
    log_verbose("r3=%12.8lf %12.8lf\n", rays[4], rays[5]);

    double bounds[6];

    rval = GREEDY_2D_generate_cut(rays, 3, f, bounds);
    abort_if(rval, "GREEDY_2D_generate_cut failed");

    for (int i = 0; i < 3; i++)
    {
        double *v = &vertices[2 * i];
        double *r = &rays[2 * i];

        v[0] = bounds[i] * r[0] + f[0];
        v[1] = bounds[i] * r[1] + f[1];
    }

CLEANUP:
    return rval;
}
